﻿EnableExplicit

;{ Macross/common stuff }
	
	; *str	:	a pointer to PB unicode string, or any memory containing unicode string
	; RETURN:	PB unicode string with ascii bytes order inside, equal to array of 1-byte chars
	Procedure.s ToAscii (*str)
		If *str 
			Protected str$ = PeekS(*str, #PB_Default, #PB_Unicode)
			Protected out$ = Space(1 + Len(str$) / SizeOf(Character))
			PokeS(@out$, str$, #PB_Default, #PB_Ascii)
			ProcedureReturn out$
		EndIf
	EndProcedure
	
;}

;{ Tesseract API }
	
	; Purebasic bindings to Tesseract engine
	; (c) Lunasole, 2016
	
	; Basic Pix. 
	; structure description seems not to be valid here, but doesn't matter (it only used by engine, not here)
	Structure PIX
		w.l         ;< width in pixels                   
		h.l			;< height in pixels                  
		d.l			;< depth in bits (bpp)               
		spp.l		;< number of samples per pixel       
		wpl.l		;< 32-bit words/line                 
					;     refcount.l  ;< reference count (1 if no clones)  
		xres.l		;< image res (ppi) in x direction    < (use 0 if unknown)  
		yres.l		;< image res (ppi) in y direction    < (use 0 if unknown)    
		
		informat.l  ;< input file format, IFF_*
		special.l	;< special instructions for I/O, etc 
		*text.Ascii	;< text string associated with pix   
		*colormap	;.PixColormap  ;< colormap (may be null)            
		*Data.Long	;< the image data                    
	EndStructure
	
	; Tesseract engine modes
	Enumeration OEM
		#OEM_TESSERACT_ONLY           ; Run Tesseract only - fastest
		#OEM_CUBE_ONLY				  ; Run Cube only - better accuracy, but slower
		#OEM_TESSERACT_CUBE_COMBINED  ; Run both and combine results - best accuracy
		#OEM_DEFAULT				  ; Specify this mode when calling init_*(),
									  ; to indicate that any of the above modes
									  ; should be automatically inferred from the
									  ; variables in the language-specific config,
									  ; command-line configs, or if not specified
									  ; in any of the above should be set to the
									  ; default OEM_TESSERACT_ONLY.
	EndEnumeration
	
	; Possible modes for page layout analysis.
	Enumeration PageSegMode
		#PSM_OSD_ONLY       ;< Orientation and script detection only.
		#PSM_AUTO_OSD		;< Automatic page segmentation with orientation and
							;< script detection. (OSD)
		#PSM_AUTO_ONLY		;< Automatic page segmentation, but no OSD, or OCR.
		#PSM_AUTO			;< Fully automatic page segmentation, but no OSD.
		#PSM_SINGLE_COLUMN	;< Assume a single column of text of variable sizes.
		#PSM_SINGLE_BLOCK_VERT_TEXT  ;< Assume a single uniform block of vertically
									 ;< aligned text.
		#PSM_SINGLE_BLOCK			 ;< Assume a single uniform block of text. (Default.)
		#PSM_SINGLE_LINE			 ;< Treat the image as a single text line.
		#PSM_SINGLE_WORD			 ;< Treat the image as a single word.
		#PSM_CIRCLE_WORD			 ;< Treat the image as a single word in a circle.
		#PSM_SINGLE_CHAR			 ;< Treat the image as a single character.
		#PSM_SPARSE_TEXT			 ;< Find as much text as possible in no particular order.
		#PSM_SPARSE_TEXT_OSD		 ;< Sparse text with orientation and script det.
		#PSM_RAW_LINE				 ;< Treat the image as a single text line, bypassing
									 ;< hacks that are Tesseract-specific.
		
		#PSM_COUNT           ;< Number of enum entries.
	EndEnumeration
	
	; some required imports. there are much more, I've just translated what was needed
	ImportC "lib\libtesseract304.lib"
		TessBaseAPIClear(*TessBaseAPI)
		TessBaseAPICreate()
		TessBaseAPIDelete(*TessBaseAPI)
		TessBaseAPIEnd(*TessBaseAPI)
		TessBaseAPIGetUTF8Text(*TessBaseAPI)
		TessBaseAPIInit1(*TessBaseAPI, datapath.p-AscII, language.p-AscII, OEM, *array_configs, configs_size)
		TessBaseAPIInit2(*TessBaseAPI, datapath.p-AscII, language.p-AscII, OEM)
		TessBaseAPIInit3(*TessBaseAPI, datapath.p-AscII, language.p-AscII)
		TessBaseAPIRecognize(*TessBaseAPI, monitor_class)
		TessBaseAPISetImage2(*TessBaseAPI, *PIX)
		TessBaseAPISetPageSegMode(*TessBaseAPI, TessPageSegMode)
		TessBaseAPIGetThresholdedImage(*TessBaseAPI)
		TessBaseAPISetVariable(*TessBaseAPI, name.p-Ascii, value.p-Ascii)
		
		; following are wrappers to leptonica lib [which is statically linked to Tesseract ^^]
		; I've exported them from Tesseract to avoid including leptonica separately here (it is needed to load image as PIX structure, used by Tesseract)
		; LeptPixRead supports lot of formats, including gif and many others
		LeptPixRead(filename.p-Ascii)
		LeptPixDestroy(hPix)
		LeptPixWritePNG(filename.p-Ascii, hPix, gamma.f)
		LeptPixReadMemPNG(*hMemory, size)
		
		
		; the following are NOT translated yet
		; TessBaseAPIAdaptToWordStr()
		; TessBaseAPIAllWordConfidences()
		; TessBaseAPIAnalyseLayout()
		; TessBaseAPIClearAdaptiveClassifier()
		; TessBaseAPIClearPersistentCache()
		; TessBaseAPIDetectOS()
		; TessBaseAPIDumpPGM()
		; TessBaseAPIFindLinesCreateBlockList()
		; TessBaseAPIGetAvailableLanguagesAsVector()
		; TessBaseAPIGetBoolVariable()
		; TessBaseAPIGetBoxText()
		; TessBaseAPIGetComponentImages()
		; TessBaseAPIGetComponentImages1()
		; TessBaseAPIGetConnectedComponents()
		; TessBaseAPIGetCubeRecoContext()
		; TessBaseAPIGetDatapath()
		; TessBaseAPIGetDawg()
		; TessBaseAPIGetDoubleVariable()
		; TessBaseAPIGetFeaturesForBlob()
		; TessBaseAPIGetHOCRText()
		; TessBaseAPIGetInitLanguagesAsString()
		; TessBaseAPIGetInputImage()
		; TessBaseAPIGetInputName()
		; TessBaseAPIGetIntVariable()
		; TessBaseAPIGetIterator()
		; TessBaseAPIGetLoadedLanguagesAsVector()
		; TessBaseAPIGetMutableIterator()
		; TessBaseAPIGetOpenCLDevice()
		; TessBaseAPIGetPageSegMode()
		; TessBaseAPIGetRegions()
		; TessBaseAPIGetSourceYResolution()
		; TessBaseAPIGetStringVariable()
		; TessBaseAPIGetStrips()
		; TessBaseAPIGetTextDirection()
		; TessBaseAPIGetTextlines()
		; TessBaseAPIGetTextlines1()
		; TessBaseAPIGetThresholdedImageScaleFactor()
		; TessBaseAPIGetUNLVText()
		; TessBaseAPIGetUnichar()
		; TessBaseAPIGetVariableAsString()
		; TessBaseAPIGetWords()
		; TessBaseAPIInit4()
		; TessBaseAPIInitForAnalysePage()
		; TessBaseAPIInitLangMod()
		; TessBaseAPIInitTruthCallback()
		; TessBaseAPIIsValidWord()
		; TessBaseAPIMeanTextConf()
		; TessBaseAPINumDawgs()
		; TessBaseAPIOem()
		; TessBaseAPIPrintVariables()
		; TessBaseAPIPrintVariablesToFile()
		; TessBaseAPIProcessPage()
		; TessBaseAPIProcessPages()
		; TessBaseAPIReadConfigFile()
		; TessBaseAPIReadDebugConfigFile()
		; TessBaseAPIRecognizeForChopTest()
		; TessBaseAPIRect()
		; TessBaseAPIRunAdaptiveClassifier()
		; TessBaseAPISetDebugVariable()
		; TessBaseAPISetDictFunc()
		; TessBaseAPISetImage()
		; TessBaseAPISetInputImage()
		; TessBaseAPISetInputName()
		; TessBaseAPISetMinOrientationMargin()
		; TessBaseAPISetOutputName()
		; TessBaseAPISetProbabilityInContextFunc()
		; TessBaseAPISetRectangle()
		; TessBaseAPISetSourceResolution()
		; TessBaseAPISetThresholder()
		; TessBoxTextRendererCreate()
		; TessChoiceIteratorConfidence()
		; TessChoiceIteratorDelete()
		; TessChoiceIteratorGetUTF8Text()
		; TessChoiceIteratorNext()
		; TessDeleteBlockList()
		; TessDeleteIntArray()
		; TessDeleteResultRenderer()
		; TessDeleteText()
		; TessDeleteTextArray()
		; TessFindRowForBox()
		; TessHOcrRendererCreate()
		; TessHOcrRendererCreate2()
		; TessMakeTBLOB()
		; TessMakeTessOCRRow()
		; TessNormalizeTBLOB()
		; TessPDFRendererCreate()
		; TessPageIteratorBaseline()
		; TessPageIteratorBegin()
		; TessPageIteratorBlockType()
		; TessPageIteratorBoundingBox()
		; TessPageIteratorCopy()
		; TessPageIteratorDelete()
		; TessPageIteratorGetBinaryImage()
		; TessPageIteratorGetImage()
		; TessPageIteratorIsAtBeginningOf()
		; TessPageIteratorIsAtFinalElement()
		; TessPageIteratorNext()
		; TessPageIteratorOrientation()
		; TessPageIteratorParagraphInfo()
		; TessResultIteratorConfidence()
		; TessResultIteratorCopy()
		; TessResultIteratorDelete()
		; TessResultIteratorGetChoiceIterator()
		; TessResultIteratorGetPageIterator()
		; TessResultIteratorGetPageIteratorConst()
		; TessResultIteratorGetUTF8Text()
		; TessResultIteratorNext()
		; TessResultIteratorSymbolIsDropcap()
		; TessResultIteratorSymbolIsSubscript()
		; TessResultIteratorSymbolIsSuperscript()
		; TessResultIteratorWordFontAttributes()
		; TessResultIteratorWordIsFromDictionary()
		; TessResultIteratorWordIsNumeric()
		; TessResultIteratorWordRecognitionLanguage()
		; TessResultRendererAddImage()
		; TessResultRendererBeginDocument()
		; TessResultRendererEndDocument()
		; TessResultRendererExtention()
		; TessResultRendererImageNum()
		; TessResultRendererInsert()
		; TessResultRendererNext()
		; TessResultRendererTitle()
		; TessTextRendererCreate()
		; TessUnlvRendererCreate()
		; TessVersion()
	EndImport
	
	
	; init tesseract with required params
	; RETURN:	handle to a Tesseract API instance on success
	Procedure TesseractInit(PageSegMode, OEM)
		Protected hApi = TessBaseAPICreate()
		Protected CFG$ = ToAscii(@"char_filter.ini")
		Protected CFG = @CFG$ ; pointer to a pointer (we passing array of strings with file names of tesseract configs to use)
		
		If Not hAPI
			Debug "Failed to create Tesseract!"
			ProcedureReturn 0
		EndIf
		
		; init tesseract to recognize numbers only
		TessBaseAPISetPageSegMode(hApi, PageSegMode) ; I'm not sure should page seg mode be set before init, or after :) so it is here twice ...
		If TessBaseAPIInit1(hAPI, GetPathPart(ProgramFilename()) + "\tessdata", "eng", OEM, @CFG, 1)
			Debug "Failed to init Tesseract!"
			TessBaseAPIEnd(hAPI)
			ProcedureReturn 0
		EndIf
		TessBaseAPISetPageSegMode(hApi, PageSegMode) ; 2nd
		ProcedureReturn hAPI
	EndProcedure
	
	; hAPI:			Tesseract API handle
	; Filename:		path to an image which has to be recognized
	; RETURN:		recognized text (numbers and . possible) or error string
	Procedure$ RecognizeText (hAPI, Filename$)
		
		; you should not just load image like that.
		; it must be preprocessed first and binarized in smart mode, for better recognition results
		; I've reached 100% recognizing quality writing custom preprocessing for specific cases, without that it was bad enough
		
		; load preprocessed image by leptonica
		;TODO: leptonica should load file from memory using LeptPixReadMemPNG(), but currently I failed to load from PB image memory :/
		Protected *TImg.Pix = LeptPixRead(Filename$)
		Protected Result$
		
		If *TImg
			; put image to engine
			TessBaseAPISetImage2(hAPI, *TImg)
			
			; 					; save internal engine image (can use for debug)
			; 					Protected *TImg_thresholded = TessBaseAPIGetThresholdedImage(hAPI)
			; 					LeptPixWritePNG ("_tess.png", *TImg_thresholded, 1.0)
			; 					LeptPixDestroy(*TImg_thresholded)
			
			; text recognize and receive recognized text
			If TessBaseAPIRecognize(hAPI, 0)
				Result$ = "[ error in Tesseract recognition ]"
			Else
				; you should also process result to remove anything you don't need
				Result$ = PeekS(TessBaseAPIGetUTF8Text(hAPI), -1, #PB_UTF8)
			EndIf
			
			; cleanup for further usage
			TessBaseAPIClear(hAPI)
			LeptPixDestroy(*TImg) ; release loaded image
		Else
			Result$ = "[ Leptonica failed to load image ]"
		EndIf
		
		ProcedureReturn Result$
	EndProcedure
;}

;{ Main }
	
	; code entry point
	Procedure Main()
		; init tesseract
		Protected hAPI = TesseractInit(#PSM_RAW_LINE, #OEM_TESSERACT_ONLY)
		
		; go
		Debug "The text on random_test.png is: " + RecognizeText(hAPI, "random_test.png")
		
		; cleanup on quit
		If hAPI
			TessBaseAPIEnd(hAPI)
		EndIf
	EndProcedure
	
;}

Main()
End


; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; CursorPosition = 318
; Folding = AAg
; EnableUnicode
; EnableXP
; CPU = 1
; SubSystem = /OPENGL
; CompileSourceDirectory
; EnableBuildCount = 16
; EnableExeConstant